#include <complex>
#include <cmath>

#include <Eigen/Dense>

#include <BufferedNode.h>
#include <Buffer.h>
#include <Vector.h>
#include <Matrix.h>
#include <Source.h>

using namespace std;
using namespace Eigen;
using namespace FD;

typedef float Real;
typedef complex<Real> Complex;

typedef float Real;
typedef complex<Real> Complex;

typedef Eigen::Matrix<Real, Dynamic, Dynamic> MatrixXr;
typedef Eigen::Matrix<Complex, Dynamic, Dynamic> MatrixXc;

typedef Eigen::Matrix<Real, Dynamic, 1> VectorXr;
typedef Eigen::Matrix<Complex, Dynamic, 1> VectorXc;

class MedianIntegrator;

DECLARE_NODE(MedianIntegrator);
/*Node
 *
 * @name MedianIntegrator
 * @category HARK:Rescue
 * @description Robust PCA
 *
 * @input_name SPEC
 * @input_type Matrix<complex<float> >
 * @input_description
 *
 * @input_name SPARSE
 * @input_type Matrix<float>
 * @input_description
 *
 * @output_name OUTPUT
 * @output_type Matrix<complex<float> >
 * @output_description
 *
 * @parameter_name NUM_CHANNEL
 * @parameter_type int
 * @parameter_value 8
 * @parameter_description
 *
 * @parameter_name LENGTH
 * @parameter_type int
 * @parameter_value 512
 * @parameter_description
 *
END*/

class MedianIntegrator : public BufferedNode
{
  int id_input_spec;
  int id_input_sparse;
  int id_output;

  int M, F;
 
public:
  MedianIntegrator(string nodeName, ParameterSet params) :
    BufferedNode(nodeName, params)
  {
    id_input_spec  = addInput("SPEC");
    id_input_sparse  = addInput("SPARSE");
    id_output = addOutput("OUTPUT");

    M = dereference_cast<int>(parameters.get("NUM_CHANNEL"));
    F = dereference_cast<int>(parameters.get("LENGTH")) / 2 + 1;
  }


  void calculate(int output_id, int count, Buffer &out)
  {
    // translate RCPtr to Eigen
    RCPtr<FD::Matrix<Complex> > p_input_spec = getInput(id_input_spec, count);
    Eigen::Map<const MatrixXc> input_spec((*p_input_spec)[0], F, M);

    RCPtr<FD::Matrix<Real> > p_input_sparse = getInput(id_input_sparse, count);
    Eigen::Map<const MatrixXr> input_sparse((*p_input_sparse)[0], F, M);

    RCPtr<FD::Matrix<Complex> > p_output(new FD::Matrix<Complex>(1, F));
    Eigen::Map<MatrixXc> output((*p_output)[0], F, 1);

    (*(outputs[id_output].buffer))[count] = p_output;

    // conduct median selection
    for (int f=0;f<F;f++) {
      VectorXr s = input_sparse.row(f);

      std::sort(s.data(), s.data() + s.size());

      if (M % 2 == 0) {
	output(f) = s(M/2-1) * input_spec(f) / abs(input_spec(f));
      } else {
      	output(f) = s(M/2) * input_spec(f) / abs(input_spec(f));
      }
    }
  }
};
